searchNodes=[{"ref":"StructAccess.html","title":"StructAccess","type":"module","doc":"Provides a standard callback implementation for the Access behaviour. Implements the following callbacks for the struct where this module is used: Access.fetch/2 Access.get/3 Access.get_and_update/3 Access.pop/2 To define these callback and include the proper behavior all you have to do is add use StructAccess to the module defining your struct. Adding use StructAccess to a module is equivalent to adding the following to that module: @behaviour Access @impl Access def fetch(struct, key), do: StructAccess.fetch(struct, key) @impl Access def get_and_update(struct, key, fun) when is_function(fun, 1) do StructAccess.get_and_update(struct, key, fun) end @impl Access def pop(struct, key, default \\ nil) do StructAccess.pop(struct, key, default) end defoverridable Access This module is simply a shortcut to avoid that boilerplate. If any of the implementations in StructAccess are not sufficient, they all can be overridden."},{"ref":"StructAccess.html#fetch/2","title":"StructAccess.fetch/2","type":"function","doc":"Retrieves the given key from the given struct. Implements the Access.fetch/2 callback."},{"ref":"StructAccess.html#get/3","title":"StructAccess.get/3","type":"function","doc":"Retrieves the given key from the given struct with a default."},{"ref":"StructAccess.html#get_and_update/3","title":"StructAccess.get_and_update/3","type":"function","doc":"Retrives the given key from the given struct and updates it at the same time. Implements the Access.get_and_update/3 callback."},{"ref":"StructAccess.html#pop/3","title":"StructAccess.pop/3","type":"function","doc":"Pops the given key from the given struct. As struct keys can&#39;t be deleted this simply sets the value of the popped key to nil. Implements the Access.pop/2 callback."},{"ref":"getting-started.html","title":"Getting Started","type":"extras","doc":"Getting Started"},{"ref":"getting-started.html#description","title":"Getting Started - Description","type":"extras","doc":"StructAccess provides a generic implementation of the Access behaviour for the module where this library is used."},{"ref":"getting-started.html#installation","title":"Getting Started - Installation","type":"extras","doc":"The package can be installed by adding struct_access to your list of dependencies in mix.exs: def deps do [ {:struct_access, &quot;~&gt; 1.0.0&quot;} ] end"},{"ref":"getting-started.html#why","title":"Getting Started - Why?","type":"extras","doc":"Why might you want to do this? So that you can take advantage of standard elixir access functions including the very useful Kernel.get_in/2 and Kernel.put_in/2 functions with nested structs."},{"ref":"getting-started.html#how-does-it-work","title":"Getting Started - How does it work?","type":"extras","doc":"When working with nested maps, the Kernel.get_in/2 function can be used to easily and safely query these maps like so: iex&gt; map = %{ animals_sounds: %{ cat: &quot;meow&quot;, dog: &quot;woof&quot; } } iex&gt; get_in(map, [:animal_sounds, :dog] &quot;woof&quot; If you try the same thing with these maps defined as structs you will see an error. iex&gt; defmodule AnimalSounds do ...&gt; defstruct cat: &quot;meow&quot;, dog: &quot;woof&quot; ...&gt; end iex&gt; defmodule Things do ...&gt; defstruct animal_sounds: %AnimalSounds{} ...&gt; end iex&gt; things = %Things{} %Things{animal_sounds: %AnimalSounds{cat: &quot;meow&quot;, dog: &quot;woof&quot;}} iex&gt; get_in(things, [:animal_sounds, :cat]) ** (UndefinedFunctionError) function AnimalSounds.fetch/2 is undefined (AnimalSounds does not implement the Access behaviour) AnimalSounds.fetch(%AnimalSounds{cat: &quot;meow&quot;, dog: &quot;woof&quot;}, :animal_sounds) (elixir) lib/access.ex:308: Access.get/3 (elixir) lib/kernel.ex:2036: Kernel.get_in/2 The solution to this is fairly straightforward, you need to add @behaviour Access to your module and then implement all of the Access callbacks. It might take you a bit to figure out, but it&#39;s straightforward to implement these callbacks so that your structs behave just like maps (with a caveat or two). But what if you have a bunch of structs in your projects that you&#39;d like to behave in this way? You&#39;ll end up repeating this implementation in each of your structs. You might even decide that you should extract that to a macro so that you can conveniently just use that macro in each of your structs removing the repitition. That&#39;s exactly what StructAccess does. Here&#39;s how to use it to make the above example just work: iex&gt; defmodule AnimalSounds do ...&gt; use StructAccess ...&gt; defstruct cat: &quot;meow&quot;, dog: &quot;woof&quot; ...&gt; end iex&gt; defmodule Things do ...&gt; use StructAccess ...&gt; defstruct animal_sounds: %AnimalSounds{} ...&gt; end iex&gt; things = %Things{} %Things{animal_sounds: %AnimalSounds{cat: &quot;meow&quot;, dog: &quot;woof&quot;}} iex&gt; get_in(things, [:animal_sounds, :cat]) &quot;meow&quot; To define these callback and include the proper behavior all you have to do is add use StructAccess to the module defining your struct. Adding use StructAccess to a module is equivalent to adding the following to that module: @behaviour Access def fetch(struct, key), do: Map.fetch(struct, key) def get(struct, key, default \\\\ nil), do: Map.get(struct, key, default) def get_and_update(struct, key, fun) when is_function(fun, 1) do current = get(struct, key) case fun.(current) do {get, update} -&gt; {get, Map.put(struct, key, update)} :pop -&gt; pop(struct, key) other -&gt; raise &quot;the given function must return a two-element tuple or :pop, got: #\\{inspect(other)}&quot; end end def pop(struct, key, default \\\\ nil) do case fetch(struct, key) do {:ok, old_value} -&gt; {old_value, Map.put(struct, key, nil)} :error -&gt; {default, struct} end end This module is simply a shortcut to avoid that boilerplate."},{"ref":"getting-started.html#caveats","title":"Getting Started - Caveats","type":"extras","doc":"Access.pop/2 One of the callbacks that needs to be implemented for the Access behavior is Access.pop/2. The intention of this function is that it removes that specified key from the map/structure, returning both that value and the updated map/structure without that key. This makes a lot of sense for a Map or Keyword, but not so much a struct, where it is impossible to remove a key. As a compromise, for cases where pop must be used, the generic implementation used in StructAccess simply sets the value of the key to be popped to nil."}]